import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;
import javax.microedition.rms.*;
import java.util.*;

public class Calculator extends MIDlet implements CommandListener
{
    TextBox mainMenu;
    static Command QUIT_CMD = new Command(Locale.RES_QUIT, Command.BACK, 4);
    static Command STOP_CMD = new Command(Locale.RES_STOP, Command.BACK, 1);
    static Command BACK_CMD = new Command(Locale.RES_BACK, Command.BACK, 4);
    static Command DONE_CMD = new Command(Locale.RES_DONE, Command.ITEM, 1);
    static Command EVAL_CMD = new Command(Locale.RES_EVALUATE, Command.ITEM, 1);
    static Command DELETE_CMD = new Command(Locale.RES_DELETE, Command.ITEM, 2);
    static Command LIST_CMD = new Command(Locale.RES_LIST, Command.ITEM, 2);
    static Command EDIT_CMD = new Command(Locale.RES_EDIT, Command.ITEM, 1);
    static Command NEW_CMD = new Command(Locale.RES_NEW, Command.ITEM, 3);
    static Command INS_CMD = new Command(Locale.RES_INSERT, Command.ITEM, 2);
    static Command HELP_CMD = new Command(Locale.RES_HELP, Command.ITEM, 2);

    //static final String TITLE = "Calculator";  // Locale.RES_TITLE

    static final int MAX_LENGTH = 256;

    static final String ENCODING = "UTF-8";

    static class Statement {
        String text;
        int    id;
        int    state;

        static final int UNKNOWN  = 0;
        static final int COMPILED = 1;
        static final int ERROR    = 2;

        byte[] pack() {
            try {
                return text.getBytes(ENCODING);
            } catch (java.io.UnsupportedEncodingException x) {
                return text.getBytes();
            }
        }

        Statement(String text) {
            this.text = text;
        }

        Statement(int id, byte[] rec) {
            this.id = id;
            try {
                text = new String(rec, ENCODING);
            } catch (java.io.UnsupportedEncodingException x) {
                text = new String(rec);
            }
        }
    }

    private void openStore() {
        try {
            store = RecordStore.openRecordStore("CALCULATOR", true);
            for (int recordId = 1, last = store.getNextRecordID(); recordId < last; recordId++) {
                byte[] record = null;
                try {
                    record = store.getRecord(recordId);
                } catch (InvalidRecordIDException x) {
                    // record was deleted
                }
                if (record == null) {
                    continue;
                }
                Statement stmt = new Statement(recordId, record);
                try {
                    Compiler compiler = new Compiler(stmt.text);
                    Compiler.Expression expr = compiler.compile();
                    if (expr instanceof Compiler.Declaration) {
                        ((Compiler.Declaration)expr).declare(bindings);
                    }
                    stmt.state = Statement.COMPILED;
                } catch (CompileError x) {
                    stmt.state = Statement.ERROR;
                }

                statements.addElement(stmt);
            }
        } catch(Exception x) {
        }
    }

    void removeStatement(int i) {
        Statement deleted = (Statement)statements.elementAt(i);
        if (stmt.id != 0 ) {
            try {
                store.deleteRecord(deleted.id);
            } catch (Exception x) {}
        }
        statements.removeElementAt(i);
        if (statements.size() != 0) {
            stmt = (Statement)statements.lastElement();
        } else {
            stmt = new Statement("");
            statements.addElement(stmt);
        }
        mainMenu.setString(stmt.text);
    }

    void editStatement(int i) {
        stmt = (Statement)statements.elementAt(i);
        mainMenu.setString(stmt.text);
    }

    void storeStatement() {
        stmt.text = mainMenu.getString();
        byte[] record = stmt.pack();
        try {
            if (stmt.id == 0) {
                stmt.id = store.addRecord(record, 0, record.length);
            } else {
                store.setRecord(stmt.id, record, 0, record.length);
            }
        } catch (Exception x) {}
    }

    private void closeStore()
    {
        if (store != null) {
            try {
                store.closeRecordStore();
                store = null;
            } catch (Exception x) {}
        }
    }

    public Calculator()
    {
        bindings = new Hashtable();
        statements = new Vector();
        openStore();
        if (statements.size() == 0) {
            stmt = new Statement("");
            statements.addElement(stmt);
        } else {
            stmt = (Statement)statements.lastElement();
        }
	mainMenu = new TextBox(Locale.RES_TITLE, stmt.text, MAX_LENGTH, TextField.ANY);

	mainMenu.addCommand(QUIT_CMD);
	mainMenu.addCommand(EVAL_CMD);
	mainMenu.addCommand(HELP_CMD);
	mainMenu.addCommand(NEW_CMD);
	mainMenu.addCommand(LIST_CMD);
	mainMenu.addCommand(DELETE_CMD);
    }

    protected void destroyApp(boolean unconditional) {
        storeStatement();
        closeStore();
        mainMenu = null;
    }

    protected  void pauseApp() {
    }

    protected void startApp()
    {
	Display.getDisplay(this).setCurrent(mainMenu);
	mainMenu.setCommandListener(this) ;
    }

    void showAlert(AlertType type, String title, String message) {
        Alert a = new Alert(title, message, null, type);
        a.setTimeout(Alert.FOREVER);
        Display.getDisplay(this).setCurrent(a, mainMenu);
    }

    void newStatement() {
        stmt = new Statement("");
        statements.addElement(stmt);
        mainMenu.setString("");
    }

    void insertText(String text) {
        mainMenu.insert(text, mainMenu.getCaretPosition());
    }

    void quit() {
        destroyApp(true);
        notifyDestroyed();
    }

    public void commandAction(Command c, Displayable d) {
        if (c == QUIT_CMD) {
            quit();
        } else if (c == LIST_CMD) {
            storeStatement();
            new StatementList(this);
        } else if (c == HELP_CMD) {
            new HelpList(this);
        } else if (c == DELETE_CMD) {
            stmt.text = "";
            mainMenu.setString("");
        } else if (c == NEW_CMD) {
            storeStatement();
            newStatement();
        } else if (c == EVAL_CMD) {
            storeStatement();
            if (stmt.text.trim().length() != 0) {
                try {
                    Compiler compiler = new Compiler(stmt.text);
                    Compiler.Expression expr = compiler.compile();
                    stmt.state = Statement.COMPILED;
                    try {
                        Real value = expr.evaluate(bindings);
                        if (expr instanceof Compiler.PlotExpression) {
                            new Plot(this, (Compiler.PlotExpression)expr);
                        } else if (expr instanceof Compiler.PlotGraphExpression) {
                            new PointGraph(this, ((Compiler.PlotGraphExpression)expr).points);
                        } else if (value != null) {
                            new ResultView(this, stmt.text, value.toString());
                        }
                    } catch (CompileError x) {
                        showAlert(AlertType.ERROR, Locale.RES_RUNTIME_ERROR, x.getMessage());
                    }
                } catch (CompileError x) {
                    stmt.state = Statement.ERROR;
                    showAlert(AlertType.ERROR, Locale.RES_COMPILE_ERROR, x.getMessage());
                }
            }
        }
    }

    RecordStore store;
    Hashtable   bindings;
    Vector      statements;
    Statement   stmt;
}