import javax.microedition.lcdui.*;

public class Graph extends Canvas implements CommandListener {
    Calculator calculator;
    Real[]    values;
    Real      beg;
    Real      end;

    Graph(Calculator calculator, Real beg, Real end, Real[] values) {
        this.calculator = calculator;
        this.beg = beg;
        this.end = end;
        this.values = values;
        setCommandListener(this);
        addCommand(Calculator.BACK_CMD);
        Display.getDisplay(calculator).setCurrent(this);
    }

    static final int RED   = 0xFF0000;
    static final int GREEN = 0x00FF00;
    static final int BLUE  = 0x0000FF;
    static final int WHITE = 0xFFFFFF;
    static final int BLACK = 0x000000;

    protected void paint(Graphics g) {
        int i;
	int width = getWidth();
	int height = getHeight();
        g.setColor(WHITE);
        g.fillRect(0, 0, width, height);

        int cx = 1;
        int cy = height-2;
        if (beg.lessThan(Real.ZERO) && end.greaterThan(Real.ZERO)) {
            cx = (int)beg.Neg().Div(end.Sub(beg)).Mul(new Real(width)).toLong();
        }
        Real min = Real.ZERO;
        Real max = Real.ZERO;
        for (i = 0; i < values.length && values[i].isNan(); i++);
        if (i < values.length) {
            min = max = values[i];
        }
        while (++i < values.length) {
            if (!values[i].isNan()) {
                if (values[i].greaterThan(max)) {
                    max = values[i];
                }
                if (values[i].lessThan(min)) {
                    min = values[i];
                }
            }
        }
        if (min.lessThan(Real.ZERO) && (max.greaterThan(Real.ZERO) || max.equalTo(Real.ZERO))) {
            cy = (int)max.Div(max.Sub(min)).Mul(height-1).toLong();
        } else if (min.equalTo(max)) {
            cy = height/2;
        }
        g.setColor(GREEN);
        g.setStrokeStyle(Graphics.SOLID);

        g.drawLine(cx, 0, cx, height-1);
        g.drawLine(cx, 0, cx-1, 3);
        g.drawLine(cx, 0, cx+1, 3);
        g.drawLine(1, cy, width, cy);
        g.drawLine(width, cy, width-3, cy-1);
        g.drawLine(width, cy, width-3, cy+1);

        Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        int fh = f.getHeight();
        g.setColor(BLACK);
        if (cy < height - 8 || cx > 8) {
            g.drawString(beg.toShortString(), 0, cy-2, Graphics.BOTTOM|Graphics.LEFT);
            g.drawString(end.toShortString(), width-1, cy-2, Graphics.BOTTOM|Graphics.RIGHT);
        }
        if (min.equalTo(max)) {
            g.drawString("0", cx+2, cy+2, Graphics.BOTTOM|Graphics.LEFT);
            int y = min.lessThan(Real.ZERO) ? height*3/4 : height/4;
            g.drawString(max.toShortString(), cx+2, y-2, Graphics.BOTTOM|Graphics.LEFT);
            g.setColor(BLUE);
            g.drawLine(0, y, width, y);
            return;
        } else {
            g.drawString(max.toShortString(), cx+3, 0, Graphics.TOP|Graphics.LEFT);
            g.drawString(min.toShortString(), cx+3, height-2, Graphics.BOTTOM|Graphics.LEFT);
        }
        g.setColor(BLUE);
        Real scale = (new Real(height-1)).Div(max.Sub(min));
        int y1 = 0;
        int x1 = 0;
        boolean gap = true;
        for (i = 0; i < values.length; i++) {
            if (values[i].isNan()) {
                gap = true;

            } else {
                int x2 = i*(width-1)/(values.length-1);
                int y2 = height-1-(int)values[i].Sub(min).Mul(scale).toLong();

                if (!gap)
                    g.drawLine(x1, y1, x2, y2);
                x1 = x2;
                y1 = y2;
                gap = false;
            }
        }
    }

    public void commandAction(Command c, Displayable d)
    {
        if (c == Calculator.BACK_CMD) {
            Display.getDisplay(calculator).setCurrent(calculator.mainMenu);
        }
    }
}