import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

public class Plot extends Form implements CommandListener, Runnable
{
    Calculator calculator;
    Gauge      progressIndicator;
    StringItem completed;
    Real      from;
    Real      till;
    Real      x;
    Real      dx;
    Real[]    values;
    int        i;
    int        percent;
    Thread     thread;
    boolean    running;
    Compiler.FunctionExpression f;

    static class DummyCanvas extends Canvas {
        protected void paint(Graphics g) {}
    }

    static Real screenWidth;
    static {
        screenWidth = new Real(new DummyCanvas().getWidth()-1);
    }

    Plot(Calculator calculator, Compiler.PlotExpression expr) {
        super(Locale.RES_PLOT);
        this.calculator = calculator;

        from = expr.from.evaluate(calculator.bindings);
        till = expr.till.evaluate(calculator.bindings);
        dx = expr.step != null
            ? expr.step.evaluate(calculator.bindings)
            : (till.Sub(from).Div(screenWidth));
        f = expr.f;

        int n = (int)till.Sub(from).Div(dx).toLong() + 1;
        if (n < 2) {
            calculator.showAlert(AlertType.ERROR, Locale.RES_ERROR, Locale.RES_BAD_INTERVAL);
            return;
        }
        values = new Real[n];
        i = 0;
        x = from;
        progressIndicator = new Gauge(Locale.RES_PROGRESS, false, n, 0);
        completed = new StringItem(Locale.RES_COMPLETED, "0%");
        append(progressIndicator);
        append(completed);

        setCommandListener(this);
        addCommand(Calculator.STOP_CMD);
        Display.getDisplay(calculator).setCurrent(this);

        running = true;
        thread = new Thread(this);
        thread.start();
    }

    public void run() {
        int n = values.length;
        try {
            while (running && i < n) {
                f.arguments[0] = x;
                values[i] = f.evaluate(calculator.bindings);
                int newPercent = i*100/n;
                if (newPercent != percent) {
                    progressIndicator.setValue(i);
                    percent = newPercent;
                    completed.setText(Integer.toString(newPercent) + '%');
                }
                x = x.Add(dx);
                i += 1;
            }
        } catch (CompileError x) {
            calculator.showAlert(AlertType.ERROR, Locale.RES_ERROR, x.getMessage());
            return;
        }
        if (i == n) {
            new Graph(calculator, from, till, values);
        }
    }

    public void commandAction(Command c, Displayable d) {
        if (c == Calculator.STOP_CMD) {
            running = false;
            Display.getDisplay(calculator).setCurrent(calculator.mainMenu);
        }
    }
}