import javax.microedition.lcdui.*;

public class PointGraph extends Canvas implements CommandListener {
    Calculator calculator;
    Point[] points;

    PointGraph(Calculator calculator, Point[] points) {
        this.calculator = calculator;
        this.points = points;
        setCommandListener(this);
        addCommand(Calculator.BACK_CMD);
        Display.getDisplay(calculator).setCurrent(this);
    }

    static final int RED   = 0xFF0000;
    static final int GREEN = 0x00FF00;
    static final int BLUE  = 0x0000FF;
    static final int WHITE = 0xFFFFFF;
    static final int BLACK = 0x000000;

    protected void paint(Graphics g) {
        int i;
	int width = getWidth();
	int height = getHeight();
        g.setColor(WHITE);
        g.fillRect(0, 0, width, height);

        int cx = 1;
        int cy = height-2;
        Real beg = points[0].x;
        Real end = beg;
        Point[] points = this.points;
        int nPoints = points.length;
        for (i = 1; i < nPoints; i++) { 
            Real x = points[i].x;
            if (x.lessThan(beg)) { 
                beg = x;
            }
            if (x.greaterThan(end)) { 
                end = x;
            }
        }
        if (beg.lessThan(Real.ZERO) && end.greaterThan(Real.ZERO)) {
            cx = (int)beg.Neg().Div(end.Sub(beg)).Mul(width).toLong();
        }
        Real min = Real.ZERO;
        Real max = Real.ZERO;
        for (i = 0; i < nPoints && points[i].y.isNan(); i++);
        if (i < nPoints) {
            min = max = points[i].y;
        }
        while (++i < nPoints) {
            Real y = points[i].y;
            if (!y.isNan()) {
                if (y.greaterThan(max)) {
                    max = y;
                }
                if (y.lessThan(min)) {
                    min = y;
                }
            }
        }
        if (min.lessThan(Real.ZERO) && (max.greaterThan(Real.ZERO) || max.equalTo(Real.ZERO))) {
            cy = (int)max.Div(max.Sub(min)).Mul(height-1).toLong();
        } else if (min.equalTo(max)) {
            cy = height/2;
        }
        g.setColor(GREEN);
        g.setStrokeStyle(Graphics.SOLID);

        g.drawLine(cx, 0, cx, height-1);
        g.drawLine(cx, 0, cx-1, 3);
        g.drawLine(cx, 0, cx+1, 3);
        g.drawLine(1, cy, width, cy);
        g.drawLine(width, cy, width-3, cy-1);
        g.drawLine(width, cy, width-3, cy+1);

        Font f = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        int fh = f.getHeight();
        g.setColor(BLACK);
        if (cy < height - 8 || cx > 8) {
            g.drawString(beg.toShortString(), 0, cy-2, Graphics.BOTTOM|Graphics.LEFT);
            g.drawString(end.toShortString(), width-1, cy-2, Graphics.BOTTOM|Graphics.RIGHT);
        }
        if (min.equalTo(max)) {
            g.drawString("0", cx+2, cy+2, Graphics.BOTTOM|Graphics.LEFT);
            int y = min.lessThan(Real.ZERO) ? height*3/4 : height/4;
            g.drawString(max.toShortString(), cx+2, y-2, Graphics.BOTTOM|Graphics.LEFT);
            g.setColor(BLUE);
            g.drawLine(0, y, width, y);
            return;
        } else {
            g.drawString(max.toShortString(), cx+3, 0, Graphics.TOP|Graphics.LEFT);
            g.drawString(min.toShortString(), cx+3, height-2, Graphics.BOTTOM|Graphics.LEFT);
        }
        g.setColor(BLUE);
        Real scaleY = (new Real(height-1)).Div(max.Sub(min));
        Real scaleX = (new Real(width-1)).Div(end.Sub(beg));
        int y1 = 0;
        int x1 = 0;
        boolean gap = true;
        for (i = 0; i < nPoints; i++) {
            if (points[i].y.isNan()) {
                gap = true;

            } else {
                int x2 = (int)points[i].x.Sub(beg).Mul(scaleX).toLong();
                int y2 = height-1-(int)points[i].y.Sub(min).Mul(scaleY).toLong();

                if (!gap) { 
                    g.drawLine(x1, y1, x2, y2);
                }
                x1 = x2;
                y1 = y2;
                gap = false;
            }
        }
    }

    public void commandAction(Command c, Displayable d)
    {
        if (c == Calculator.BACK_CMD) {
            Display.getDisplay(calculator).setCurrent(calculator.mainMenu);
        }
    }
}